@@ -13,7 +13,7 @@ from pkg_resources import resource_string
 
 # pylint: disable-msg=R0903,C0111
 class SutekhInfo(object):
-    VERSION = (0, 8, 4, 'final', 0)
+    VERSION = (0, 8, 5, 'final', 0)
     BASE_VERSION_STR = '.'.join([str(x) for x in VERSION[:3]])
     VERSION_STR = {
         'final': BASE_VERSION_STR,
@@ -82,11 +82,12 @@ class SutekhInfo(object):
     ]
 
     INSTALL_REQUIRES = [
-        'SQLObject >= 0.9.0, < 1.6',  # fetching the dev version requires svn
+        'SQLObject >= 0.9.0, < 1.7',  # fetching the dev version requires svn
                            # (which is a bit crazy as an install requirement)
         'PyProtocols',
         'ply',
         'configobj',
+        'simplejson',
     ]
 
     # Install these manually
@@ -127,6 +128,10 @@ class SutekhInfo(object):
             'License :: OSI Approved :: BSD License',
             'http://www.voidspace.org.uk/python/configobj.html#license',
             'New-BSD license'),
+        'simplejson': (
+            'License :: OSI Approved :: MIT License',
+            'https://raw.githubusercontent.com/simplejson/simplejson/master/LICENSE.txt',
+            'MIT license'),
         'setuptools': (
             'License :: OSI Approved :: Python Software Foundation License',
             'http://www.python.org/psf/license/',
