@@ -7,9 +7,11 @@
 
 """Base class for Sutekh Frames"""
 
-import gtk, gobject
+import gtk
+import gobject
 
-class BasicFrame(gtk.Frame, object):
+
+class BasicFrame(gtk.Frame):
     # pylint: disable-msg=R0904
     # gtk.Widget, so lotes of public methods
     """The basic, blank frame for sutekh.
@@ -19,13 +21,16 @@ class BasicFrame(gtk.Frame, object):
        setting the active hint.
        """
 
-    aDragTargets = [ ('STRING', 0, 0),
-            ('text/plain', 0, 0) ]
+    aDragTargets = [('STRING', 0, 0), ('text/plain', 0, 0)]
+
+    _cModelType = None
 
     def __init__(self, oMainWindow):
         super(BasicFrame, self).__init__()
         self._oMainWindow = oMainWindow
+        self._aPlugins = []
         self.set_name("blank frame")
+        self._iId = 0
 
         # Ensure new panes aren't completely hidden
 
@@ -51,7 +56,12 @@ class BasicFrame(gtk.Frame, object):
 
         self._oTitle.connect('drag-data-received', self.drag_drop_handler)
         self._oTitle.connect('drag-data-get', self.create_drag_data)
-        self.set_drag_handler()
+        self._oTitle.connect('button-press-event', self.minimize_to_toolbar)
+        self._oTitle.connect_after('drag_begin', self.make_drag_icon)
+        self.set_drag_handler(self._oView)
+        self.set_drop_handler(self._oView)
+
+        self.set_unique_id()
 
     # pylint: disable-msg=W0212
     # explicitly allow access to these values via thesep properties
@@ -63,19 +73,51 @@ class BasicFrame(gtk.Frame, object):
     view = property(fget=lambda self: self._oView,
             doc="Associated View Object")
     menu = property(fget=lambda self: None, doc="Frame's menu")
+    plugins = property(fget=lambda self: self._aPlugins,
+            doc="Plugins enabled for this frame.")
+    pane_id = property(fget=lambda self: self._iId,
+            doc="ID number for this pane (should be unique)")
+    config_frame_id = property(fget=lambda self: "pane%s" % (self._iId,),
+            doc="Config frame id for this pane")
     # pylint: enable-msg=W0212
 
+    def set_unique_id(self):
+        """Set a unique id for this pane"""
+        # the [0] is to a) ensure max is never over an empty set and
+        # b) that the id is positive
+        self._iId = max(self._oMainWindow.get_pane_ids() + [0]) + 1
+
+    def init_plugins(self):
+        """Loop through the plugins, and enable those appropriate for us."""
+        for cPlugin in \
+                self._oMainWindow.plugin_manager.get_card_list_plugins():
+            self._aPlugins.append(cPlugin(self._oController.view,
+                self._oController.view.get_model(), self._cModelType))
+
     def set_title(self, sTitle):
         """Set the title of the pane to sTitle"""
         self._oTitleLabel.set_markup(gobject.markup_escape_text(sTitle))
 
-    def set_drag_handler(self):
-        """Setup the appropriate drag-n-drop handler for the view"""
-        self._oView.drag_dest_set(gtk.DEST_DEFAULT_ALL,
+    def set_id(self, iNewId):
+        """Set the id of the pane to the correct value"""
+        self._iId = iNewId
+
+    def set_drop_handler(self, oWidget):
+        """Setup the frame drop handler on the widget"""
+        oWidget.drag_dest_set(gtk.DEST_DEFAULT_ALL,
                 self.aDragTargets,
                 gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
-        self._oView.connect('drag-data-received', self.drag_drop_handler)
-        self._oView.connect('drag-motion', self.drag_motion)
+        oWidget.connect('drag-data-received', self.drag_drop_handler)
+        oWidget.connect('drag-motion', self.drag_motion)
+
+    def set_drag_handler(self, oWidget):
+        """Enable dragging of the frame via given widget"""
+        oWidget.drag_source_set(
+                gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
+                self.aDragTargets,
+                gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
+        oWidget.connect('drag-data-get', self.create_drag_data)
+        oWidget.connect_after('drag_begin', self.make_drag_icon)
 
     def set_focus_handler(self, oFunc):
         """Set the button press handler for the frame"""
@@ -88,7 +130,7 @@ class BasicFrame(gtk.Frame, object):
     def do_swap(self, aData):
         """Swap this pane with the relevant pane"""
         # We swap ourself with the other pane
-        oOtherFrame = self._oMainWindow.find_pane_by_name(aData[1])
+        oOtherFrame = self._oMainWindow.find_pane_by_id(int(aData[1]))
         if oOtherFrame:
             self._oMainWindow.swap_frames(self, oOtherFrame)
             return True
@@ -97,29 +139,53 @@ class BasicFrame(gtk.Frame, object):
     def do_dragged_card_set(self, aData):
         """Replace this pane with the relevant card set"""
         # We replace otherselves with the card set
-        oFrame = self._oMainWindow.find_pane_by_name(aData[1])
-        if not oFrame:
-            # Card set is not already open
-            self._oMainWindow.replace_with_physical_card_set(aData[1], self)
-            return True
+        # Check that we're not the same card set already
+        if self.is_card_set(aData[1]):
+            return False
+        # Open the card set
+        self._oMainWindow.replace_with_physical_card_set(aData[1], self)
+        return True
+
+    # pylint: disable-msg=R0201
+    # Methods so sub-classes can override them
+    def is_card_set(self, _sSetName):
+        """Returns true if we're a copy of the given card set"""
         return False
 
+    def get_menu_name(self):
+        """Return the key into the menu dictionary in the main window"""
+        return None
+
+    # pylint: enable-msg=R0201
+
     def cleanup(self):
         """Hook for cleanup actions when the frame is removed."""
-        pass
+        # do per-plugin cleanup
+        for oPlugin in self._aPlugins:
+            oPlugin.cleanup()
+        # Don't hold references to the plugins
+        self._aPlugins = []
+        # Cleanup after the menu if required
+        if self.menu and hasattr(self.menu, 'cleanup'):
+            self.menu.cleanup()
 
     def reload(self):
         """Reload frame contents"""
         pass
 
     def do_queued_reload(self):
-        """Do a defferred reload if one was set earlier"""
+        """Do a deferred reload if one was set earlier"""
         self._bNeedReload = False
 
     def queue_reload(self):
         """Queue a reload for later - needed so reloads can happen after
            database update signals."""
         self._bNeedReload = True
+        # Add timeout so reload is called in the near future
+        # As we're single threaded, this won't be executred until we're
+        # done with the current main_loop iteration, so it will only
+        # happen after all our current processing.
+        gobject.timeout_add(20, self._oMainWindow.do_all_queued_reloads)
 
     def update_to_new_db(self):
         """Re-associate internal data against the database.
@@ -131,6 +197,7 @@ class BasicFrame(gtk.Frame, object):
 
     def close_frame(self):
         """Close the frame"""
+        self._oMainWindow.config_file.clear_frame_profile(self.config_frame_id)
         self._oMainWindow.remove_frame(self)
         self.destroy()
 
@@ -177,7 +244,7 @@ class BasicFrame(gtk.Frame, object):
                 fg[NORMAL] = "%(colour)s"
                 }
             widget "%(path)s" style "internal_sutekh_hlstyle"
-            """ % { 'colour' : sColour, 'path' : self._oTitleLabel.path() }
+            """ % {'colour': sColour, 'path': self._oTitleLabel.path()}
             gtk.rc_parse_string(sStyleInfo)
             # We use gtk's style machinery to do the update.
             # This seems the only way of ensuring we will actually do
@@ -190,50 +257,61 @@ class BasicFrame(gtk.Frame, object):
         self._oTitleLabel.set_name('frame_title')
         self._oTitle.set_name('frame_title')
 
-    # pylint: disable-msg=W0613, R0913
-    # function signature requires these arguments
-    def close_menu_item(self, oMenuWidget):
+    def close_menu_item(self, _oMenuWidget):
         """Handle close requests from the menu."""
         self.close_frame()
 
-    def call_focus(self, oWidget, oEvent, oFocusFunc):
+    def call_focus(self, _oWidget, oEvent, oFocusFunc):
         """Call MultiPaneWindow focus handler for button events"""
         oFocusFunc(self, oEvent, self)
         return False
 
-    def drag_drop_handler(self, oWindow, oDragContext, iXPos, iYPos,
-            oSelectionData, oInfo, oTime):
+    # pylint: disable-msg=R0913
+    # function signature requires all these arguments
+    def drag_drop_handler(self, _oWindow, oDragContext, _iXPos, _iYPos,
+            oSelectionData, _oInfo, oTime):
         """Handle panes being dragged onto this one.
 
            Allows panes to be sapped by dragging 'n dropping."""
+        bDragRes = True
         if not oSelectionData and oSelectionData.format != 8:
-            oDragContext.finish(False, False, oTime)
+            bDragRes = False
         else:
-            aData =  oSelectionData.data.splitlines()
+            aData = oSelectionData.data.splitlines()
             if aData[0] == 'Sutekh Pane:':
-                if self.do_swap(aData):
-                    oDragContext.finish(True, False, oTime)
-                else:
-                    oDragContext.finish(False, False, oTime)
+                if not self.do_swap(aData):
+                    bDragRes = False
             elif aData[0] == 'Card Set:':
-                if self.do_dragged_card_set(aData):
-                    oDragContext.finish(True, False, oTime)
-                else:
-                    oDragContext.finish(False, False, oTime)
+                if not self.do_dragged_card_set(aData):
+                    bDragRes = False
             else:
-                oDragContext.finish(False, False, oTime)
+                bDragRes = False
+        oDragContext.finish(bDragRes, False, oTime)
 
-    def create_drag_data(self, oBtn, oContext, oSelectionData, oInfo, oTime):
+    def create_drag_data(self, _oBtn, _oContext, oSelectionData, _oInfo,
+            _oTime):
         """Fill in the needed data for drag-n-drop code"""
-        sData = 'Sutekh Pane:\n' + self.title
+        sData = 'Sutekh Pane:\n%s' % self.pane_id
         oSelectionData.set(oSelectionData.target, 8, sData)
 
     # pylint: disable-msg=R0201
-    # needs to be a mthod, as children can override this if needed
-    def drag_motion(self, oWidget, oDrag_context, iXPos, iYPos, oTimestamp):
+    # needs to be a method, as children can override this if needed
+    def drag_motion(self, _oWidget, oDrag_context, _iXPos, _iYPos,
+            _oTimestamp):
         """Show proper icon during drag-n-drop actions."""
         if 'STRING' in oDrag_context.targets:
             oDrag_context.drag_status(gtk.gdk.ACTION_COPY)
             return True
         return False
-    # pylint: enable-msg=R0913, W0613
+
+    def minimize_to_toolbar(self, _oWidget, oEvent):
+        """Minimize the frame to the toolbar on double-click."""
+        # pylint: disable-msg=W0212
+        # We need to access _2BUTTON_PRESS
+        if oEvent.type == gtk.gdk._2BUTTON_PRESS:
+            self._oMainWindow.minimize_to_toolbar(self)
+
+    def make_drag_icon(self, oWidget, _oDragContext):
+        """Create an icon for dragging the pane from the titlebar"""
+        oDrawable = self._oTitleLabel.get_snapshot(None)
+        oWidget.drag_source_set_icon(oDrawable.get_colormap(), oDrawable)
