@@ -9,19 +9,18 @@
 
 import gtk
 from sqlobject import sqlhub
-from sutekh.core.CardLookup import LookupFailed
-from sutekh.core.SutekhObjects import IPhysicalCardSet
-from sutekh.gui.SutekhDialog import do_complaint_error, do_complaint_warning
+from sutekh.gui.SutekhDialog import do_complaint_error
 from sutekh.gui.SutekhFileWidget import ImportDialog
 from sutekh.gui.GuiDBManagement import refresh_ww_card_list
-from sutekh.gui.CardSetManagementController import reparent_card_set
-from sutekh.io.XmlFileHandling import PhysicalCardXmlFile, \
-        PhysicalCardSetXmlFile, AbstractCardSetXmlFile
+from sutekh.gui.GuiCardSetFunctions import import_cs
+from sutekh.gui.ProfileManagement import ProfileMngDlg
 from sutekh.io.IdentifyXMLFile import IdentifyXMLFile
-from sutekh.core.CardSetUtilities import delete_physical_card_set, \
-        find_children
+from sutekh.io.AbstractCardSetParser import AbstractCardSetParser
+from sutekh.io.PhysicalCardParser import PhysicalCardParser
+from sutekh.io.PhysicalCardSetParser import PhysicalCardSetParser
 from sutekh.gui.SutekhMenu import SutekhMenu
 
+
 class MainMenu(SutekhMenu):
     """The Main application Menu.
 
@@ -36,7 +35,8 @@ class MainMenu(SutekhMenu):
         self.__oConfig = oConfig
         self.__create_file_menu()
         self.__create_pane_menu()
-        self.create_plugins_menu('Plugins', oWindow)
+        self.__create_rulebook_menu()
+        self.add_plugins_to_menus(oWindow)
         self.__create_help_menu()
         oWindow.add_to_menu_list("White Wolf Card List",
                 self.physical_cl_set_sensitive)
@@ -53,6 +53,8 @@ class MainMenu(SutekhMenu):
         """Create the File Menu"""
         oMenu = self.create_submenu(self, "_File")
         # items
+
+        self.create_submenu(oMenu, 'Backup')
         oImport = self.create_submenu(oMenu, "Import Card Set")
 
         self.create_menu_item("Load Saved Card Set from File", oImport,
@@ -60,18 +62,17 @@ class MainMenu(SutekhMenu):
 
         oMenu.add(gtk.SeparatorMenuItem())
 
-        if sqlhub.processConnection.uri() != "sqlite:///:memory:":
-            # Need to have memory connection available for this
-            self.create_menu_item("Import new White Wolf Card List and"
-                    " rulings", oMenu, self.do_import_new_card_list)
-        self.create_menu_item('Download VTES icons', oMenu,
-                self.download_icons)
+        oDownloadMenu = self.create_submenu(oMenu, 'Data Downloads')
+        self.__add_download_menu(oDownloadMenu)
+
         oMenu.add(gtk.SeparatorMenuItem())
 
         oPrefsMenu = self.create_submenu(oMenu, 'Preferences')
-
         self.__add_prefs_menu(oPrefsMenu)
 
+        self.create_menu_item('Manage Profiles', oMenu,
+                self.do_manage_profiles)
+
         self.create_menu_item('Save Current Pane Set', oMenu,
                 self.do_save_pane_set)
 
@@ -88,6 +89,15 @@ class MainMenu(SutekhMenu):
         """Add the File Preferences menu"""
         self._dMenus["File Preferences"] = oPrefsMenu
 
+        oNameDisplay = gtk.CheckMenuItem('Use "path of ..., the" name display')
+        oNameDisplay.set_inconsistent(False)
+        if self.__oConfig.get_postfix_the_display():
+            oNameDisplay.set_active(True)
+        else:
+            oNameDisplay.set_active(False)
+        oNameDisplay.connect('activate', self.do_postfix_the_display)
+        oPrefsMenu.add(oNameDisplay)
+
         oSaveOnExit = gtk.CheckMenuItem('Save Pane Set on Exit')
         oSaveOnExit.set_inconsistent(False)
         if self.__oConfig.get_save_on_exit():
@@ -115,12 +125,21 @@ class MainMenu(SutekhMenu):
         oSaveOnExit.connect('activate', self.do_toggle_save_window_size)
         oPrefsMenu.add(oSaveWinSize)
 
+    def __add_download_menu(self, oDownloadMenu):
+        """Add the File Download menu"""
+        if sqlhub.processConnection.uri() != "sqlite:///:memory:":
+            # Need to have memory connection available for this
+            self.create_menu_item("Import new White Wolf Card List and"
+                    " rulings", oDownloadMenu, self.do_import_new_card_list)
+        self.create_menu_item('Download VTES icons', oDownloadMenu,
+                self.download_icons)
+
     def __create_pane_menu(self):
         """Create the 'Pane Actions' menu"""
-        oMenu = self.create_submenu(self, 'Pane A_ctions')
+        oMenu = self.create_submenu(self, '_Pane Actions')
 
-        self.create_menu_item("Equalize pane sizes", oMenu,
-                self.equalize_panes)
+        self.create_menu_item("Equalize _pane sizes", oMenu,
+                self.equalize_panes, "<Ctrl>p")
 
         self.__oAddHorzPane = self.create_menu_item("Split current pane"
                 " _horizontally (|)", oMenu, self.add_pane_horizontal)
@@ -177,6 +196,11 @@ class MainMenu(SutekhMenu):
                 self._oMainWindow.replace_with_pcs_list)
         self.__oReplacePCSListPane.set_sensitive(True)
 
+    def __create_rulebook_menu(self):
+        """Create the menu for rulebook items"""
+        # setup sub menu
+        self.create_menu_item_with_submenu(self, "_Rulebook")
+
     def __create_help_menu(self):
         """Create the menu for help items"""
         # setup sub menu
@@ -190,7 +214,7 @@ class MainMenu(SutekhMenu):
                 self._oMainWindow.show_last_help)
         self.oHelpLast.set_sensitive(False)
         self.create_menu_item("About Sutekh", oMenu,
-                self._oMainWindow.show_about_dialog, '<Ctrl>A')
+                self._oMainWindow.show_about_dialog)
 
     # pylint: enable-msg=W0201
 
@@ -218,7 +242,7 @@ class MainMenu(SutekhMenu):
         self.__oAddPCLPane.set_sensitive(bValue)
 
     def set_split_vertical_active(self, bValue):
-        """Set the split veritcal pane option to bValue"""
+        """Set the split vertical pane option to bValue"""
         self.__oAddVertPane.set_sensitive(bValue)
 
     def set_split_horizontal_active(self, bValue):
@@ -233,9 +257,7 @@ class MainMenu(SutekhMenu):
         """Show the Sutekh Tutorial"""
         self._oMainWindow.show_manual(oMenuWidget, self.oHelpLast)
 
-    # pylint: disable-msg=W0613
-    # oWidget + oMenuWidget required by function signature
-    def do_import_card_set(self, oWidget):
+    def do_import_card_set(self, _oWidget):
         """Import a card set from a XML File."""
         oFileChooser = ImportDialog("Select Card Set(s) to Import",
                 self._oMainWindow)
@@ -245,103 +267,73 @@ class MainMenu(SutekhMenu):
         if sFileName is not None:
             oIdParser = IdentifyXMLFile()
             oIdParser.id_file(sFileName)
-            aChildren = []
             if oIdParser.type == 'PhysicalCardSet' or \
                     oIdParser.type == 'AbstractCardSet' or \
                     oIdParser.type == 'PhysicalCard':
-                if oIdParser.exists:
-                    iResponse = do_complaint_warning("This would delete the"
-                            " existing CardSet %s" % oIdParser.name)
-                    if iResponse == gtk.RESPONSE_CANCEL:
-                        return
-                    else:
-                        # Delete the card set
-                        oCS = IPhysicalCardSet(oIdParser.name)
-                        aChildren = find_children(oCS)
-                        # we need to make sure we do the right
-                        # thing for any children of the card set.
-                        # delete_physical_card_set will reparent them,
-                        # but we need to be able to re-reparent them to
-                        # the new card set, so re-importing a card set
-                        # with children added does the expected thing
-                        delete_physical_card_set(oIdParser.name)
-                oFrame = self._oMainWindow.add_pane_end()
-                try:
-                    if oIdParser.type == "AbstractCardSet":
-                        oFile = AbstractCardSetXmlFile(sFileName,
-                                oLookup=self._oMainWindow.cardLookup)
-                        aMessages = oFile.read(bIgnoreWarnings=False)
-                    elif oIdParser.type == 'PhysicalCardSet':
-                        oFile = PhysicalCardSetXmlFile(sFileName,
-                                oLookup=self._oMainWindow.cardLookup)
-                        aMessages = oFile.read(bIgnoreWarnings=False)
-                    else:
-                        # Old style PhysicalCard list
-                        oFile = PhysicalCardXmlFile(sFileName,
-                                oLookup=self._oMainWindow.cardLookup)
-                        aMessages = oFile.read(bIgnoreWarnings=False)
-                    if aMessages:
-                        sMsg = "The following warnings were reported:\n%s" % \
-                                "\n".join(aMessages)
-                        do_complaint_warning(sMsg)
-                    # Reparent any children
-                    oCS = IPhysicalCardSet(oIdParser.name)
-                    # We worry about loops, since the recreated card set
-                    # may not have the same parent as before
-                    for oChildCS in aChildren:
-                        reparent_card_set(oChildCS, oCS)
-                    self._oMainWindow.replace_with_physical_card_set(
-                            oIdParser.name, oFrame)
-                except LookupFailed:
-                    # Remove window, since we didn't succeed
-                    # Should this dialog be here?
-                    do_complaint_error("Import failed. Unable to find "
-                            "matches for all the cards in the cardset.")
-                    self._oMainWindow.remove_frame(oFrame)
+                fIn = file(sFileName, 'rU')
+                if oIdParser.type == "AbstractCardSet":
+                    oParser = AbstractCardSetParser()
+                elif oIdParser.type == 'PhysicalCardSet':
+                    oParser = PhysicalCardSetParser()
+                else:
+                    # Old style PhysicalCard list
+                    oParser = PhysicalCardParser()
+                import_cs(fIn, oParser, self._oMainWindow)
             else:
                 do_complaint_error("File is not a CardSet XML File.")
 
-    def do_import_new_card_list(self, oWidget):
+    def do_import_new_card_list(self, _oWidget):
         """Refresh the WW card list and rulings files."""
         if refresh_ww_card_list(self._oMainWindow):
             self._oMainWindow.reload_all()
 
-    def do_save_pane_set(self, oWidget):
+    def do_postfix_the_display(self, _oWidget):
+        """Save the current pane layout"""
+        bChoice = not self.__oConfig.get_postfix_the_display()
+        self.__oConfig.set_postfix_the_display(bChoice)
+
+    def do_manage_profiles(self, _oWidget):
+        """Display the Profile management dialog"""
+        oDlg = ProfileMngDlg(self._oMainWindow, self.__oConfig)
+        oDlg.run()
+        oDlg.destroy()
+
+    def do_save_pane_set(self, _oWidget):
         """Save the current pane layout"""
         self._oMainWindow.save_frames()
 
-    def do_toggle_save_on_exit(self, oWidget):
+    def do_toggle_save_on_exit(self, _oWidget):
         """Toggle the 'Save Pane layout on exit' option"""
         bChoice = not self.__oConfig.get_save_on_exit()
         self.__oConfig.set_save_on_exit(bChoice)
         # gtk can handle the rest for us
 
-    def do_toggle_save_precise_pos(self, oWidget):
+    def do_toggle_save_precise_pos(self, _oWidget):
         """Toggle save precise pane positions option"""
         bChoice = not self.__oConfig.get_save_precise_pos()
         self.__oConfig.set_save_precise_pos(bChoice)
 
-    def do_toggle_save_window_size(self, oWidget):
+    def do_toggle_save_window_size(self, _oWidget):
         """Toggle save window size option"""
         bChoice = not self.__oConfig.get_save_window_size()
         self.__oConfig.set_save_window_size(bChoice)
 
-    def equalize_panes(self, oMenuWidget):
+    def equalize_panes(self, _oWidget):
         """Ensure all panes have the same width allocated."""
         self._oMainWindow.set_all_panes_equal()
 
-    def add_pane_horizontal(self, oMenuWidget):
-        """Split the current pane horizonitally and add a new pane."""
+    def add_pane_horizontal(self, _oWidget):
+        """Split the current pane horizontally and add a new pane."""
         self._oMainWindow.add_pane(False)
 
-    def add_pane_vertical(self, oMenuWidget):
+    def add_pane_vertical(self, _oWidget):
         """Split the current pane vertically and add a new pane."""
         self._oMainWindow.add_pane(True)
 
-    def do_restore(self, oMenuWidget):
+    def do_restore(self, _oWidget):
         """Restore the pane layout from the config file."""
         self._oMainWindow.restore_from_config()
 
-    def download_icons(self, oMenuWidget):
+    def download_icons(self, _oWidget):
         """Call on the icon manager to download the icons."""
         self._oMainWindow.icon_manager.download_icons()
