@@ -14,6 +14,7 @@ import string
 # pylint: enable-msg=W0402
 from logging import Handler
 
+
 class SutekhLogHandler(Handler, object):
     """Base class for loggers to talk to the dialog"""
     # We explicitly inherit from object, since Handler is a classic class
@@ -25,12 +26,11 @@ class SutekhLogHandler(Handler, object):
         """point to the progress dialog"""
         self.oDialog = oDialog
 
-    # pylint: disable-msg=W0613
-    # oRecord required by function signature
-    def emit(self, oRecord):
+    def emit(self, _oRecord):
         """Default emit handler"""
         pass
 
+
 class SutekhHTMLLogHandler(SutekhLogHandler):
     """Logging class for cardlist and rulings parser.
 
@@ -47,7 +47,7 @@ class SutekhHTMLLogHandler(SutekhLogHandler):
            Skip difficult cases (The X, non-ascii characters, etc.)
            """
         if self.oDialog is None:
-            return # No point
+            return  # No point
         sString = oRecord.getMessage()
         # We skip considering the 'The' cases, as that gets messy
         if sString.startswith('Card: The '):
@@ -64,10 +64,11 @@ class SutekhHTMLLogHandler(SutekhLogHandler):
                 sStart[1] not in string.ascii_uppercase:
             return
         # This is now safe, if not a particularly good idea
-        iPos = ord(sStart[0])*26+ord(sStart[1])-(ord('A')*27)
-        fBarPos = iPos/float(26*26)
+        iPos = ord(sStart[0]) * 26 + ord(sStart[1]) - (ord('A') * 27)
+        fBarPos = iPos / float(26 * 26)
         self.oDialog.update_bar(fBarPos)
 
+
 class SutekhCountLogHandler(SutekhLogHandler):
     """LogHandler class for dealing with database upgrade messages.
 
@@ -84,16 +85,15 @@ class SutekhCountLogHandler(SutekhLogHandler):
         self.fTot = float(iTot)
         self.iCount = 0
 
-    # pylint: disable-msg=W0613
-    # oRecord required by function signature
-    def emit(self, oRecord):
+    def emit(self, _oRecord):
         """Handle a emitted signal, updating the progress count."""
         if self.oDialog is None:
-            return # No point
+            return  # No point
         self.iCount += 1
-        fBarPos = self.iCount/self.fTot
+        fBarPos = self.iCount / self.fTot
         self.oDialog.update_bar(fBarPos)
 
+
 class ProgressDialog(gtk.Window):
     # pylint: disable-msg=R0904
     # gtk.Widget, so many public methods
@@ -118,6 +118,7 @@ class ProgressDialog(gtk.Window):
         oAlign.add(self.oProgressBar)
         self.oVBox.pack_end(oAlign)
         self.add(self.oVBox)
+        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_UTILITY)
         self.show_all()
         self.set_modal(True)
 
@@ -141,14 +142,15 @@ class ProgressDialog(gtk.Window):
         # Progress bar doesn't like values < 0.0 or > 1.0
         if fStep > 1.0:
             self.oProgressBar.set_fraction(1.0)
+            self.oProgressBar.set_text('100% complete')
         elif fStep < 0.0:
             self.oProgressBar.set_fraction(0.0)
+            self.oProgressBar.set_text(' 0% complete')
         else:
             self.oProgressBar.set_fraction(fStep)
+            self.oProgressBar.set_text('%2.0f%% complete' % (fStep * 100))
         # Since Sutekh is single threaded, and some IO bound
         # task has the gtk.main loop process, we fudge the
         # required behaviour to get the update drawn
         while gtk.events_pending():
             gtk.main_iteration()
-
-
