@@ -4,14 +4,15 @@
 # Dialog to display deck analysis software
 # Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>,
 # GPL - see COPYING for details
-"""Calculate probailibities for drawing the current selection."""
+"""Calculate probabilities for drawing the current selection."""
 
 import gtk
 from copy import copy
-from sutekh.core.SutekhObjects import PhysicalCardSet, \
-        IAbstractCard
-from sutekh.gui.PluginManager import CardListPlugin
-from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
+from sutekh.core.SutekhObjects import PhysicalCardSet, IAbstractCard
+from sutekh.SutekhUtility import is_crypt_card
+from sutekh.gui.PluginManager import SutekhPlugin
+from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error, \
+        do_complaint_warning
 from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
 
 
@@ -21,13 +22,14 @@ def _choose(iChoices, iTotal):
     if iChoices > 0:
         iDenom = iChoices
     else:
-        return 1 # 0!/0! = 1, since 0! = 1
+        return 1  # 0!/0! = 1, since 0! = 1
     iNumerator = iTotal
     for iNum in range(1, iChoices):
         iNumerator *= (iTotal - iNum)
         iDenom *= iNum
     return iNumerator / iDenom
 
+
 def _gen_choice_list(dSelectedCounts):
     """Recursively generate all possible choices"""
     aList = []
@@ -38,16 +40,17 @@ def _gen_choice_list(dSelectedCounts):
         dNew = copy(dSelectedCounts)
         del dNew[sThisItem]
         aSubLists = _gen_choice_list(dNew)
-        for iChoice in range(dSelectedCounts[sThisItem]+1):
+        for iChoice in range(dSelectedCounts[sThisItem] + 1):
             for aChoices in aSubLists:
                 aThisList = [iChoice]
                 aThisList.extend(aChoices)
                 aList.append(aThisList)
     else:
-        aList = [[x] for x in range(dSelectedCounts[sThisItem]+1)]
-    aList.sort(key=lambda x: [sum(x)]+x)
+        aList = [[x] for x in range(dSelectedCounts[sThisItem] + 1)]
+    aList.sort(key=lambda x: [sum(x)] + x)
     return aList
 
+
 def _multi_hyper_prob(aFound, iDraws, aObjects, iTotal):
     """Multivariate hypergeometric probability:
 
@@ -90,6 +93,7 @@ def _multi_hyper_prob(aFound, iDraws, aObjects, iTotal):
         fNumerator *= _choose(iRemFound, iRemObjects)
     return fNumerator / fDemon
 
+
 def _hyper_prob_at_least(aFound, iDraws, aObjects, iTotal, iCurCol=0):
     """Returns the probablity of drawing at least aFound from aObjects objects
        of interest from iTotal objects in iDraw draws."""
@@ -110,28 +114,31 @@ def _hyper_prob_at_least(aFound, iDraws, aObjects, iTotal, iCurCol=0):
             fProb += _multi_hyper_prob(aThisFound, iDraws, aObjects, iTotal)
     return fProb
 
-class CardDrawSimPlugin(CardListPlugin):
+
+class CardDrawSimPlugin(SutekhPlugin):
     """Displays the probabilities for drawing cards from the current
        selection."""
     # pylint: disable-msg=R0902
-    # we use a lot of attributes to parse the data around
-    dTableVersions = {PhysicalCardSet : [3, 4, 5]}
-    aModelsSupported = [PhysicalCardSet]
+    # we use a lot of attributes to pass the data around
+    dTableVersions = {PhysicalCardSet: (4, 5, 6)}
+    aModelsSupported = (PhysicalCardSet,)
 
     def get_menu_item(self):
-        """Register on the 'Plugins' menu"""
+        """Register on the 'Analyze' menu"""
         if not self.check_versions() or not self.check_model_type():
             return None
         oCardDraw = gtk.MenuItem("Card Draw probabilities")
         oCardDraw.connect("activate", self.activate)
-        return ('Plugins', oCardDraw)
+        return ('Analyze', oCardDraw)
 
-    # pylint: disable-msg=W0613, W0201
-    # W0613 - oWidget has to be here, although it's unused
+    # pylint: disable-msg=W0201
     # W0201 - we define lots of things here, rather than __init__, since this
     # is the plugin's entry point, and they need to reflect the current state
-    def activate(self, oWidget):
+    def activate(self, _oWidget):
         """Create the actual dialog, and populate it."""
+        if not self.check_cs_size('Card Draw probabilities', 500):
+            return
+
         self.iTotal = 0
         self.dSelectedCounts = {}
         self.iSelectedCount = 0
@@ -151,6 +158,13 @@ class CardDrawSimPlugin(CardListPlugin):
 
         self._setup_cardlists(aSelectedCards, bCrypt)
 
+        # Query user if we have zero counts
+        if 0 in self.dSelectedCounts.values():
+            iRes = do_complaint_warning("Selected cards include cards with"
+                    " a count of 0.\nDo you want to continue?")
+            if iRes == gtk.RESPONSE_CANCEL:
+                return
+
         oMainTitle = gtk.Label()
         if bCrypt:
             oMainTitle.set_markup('<b>Crypt:</b> Drawing from <b>%d</b> '
@@ -164,14 +178,14 @@ class CardDrawSimPlugin(CardListPlugin):
             # Look 15 cards into the deck by default, seems good start
             self.iNumSteps = min(8, self.iTotal - self.iOpeningDraw)
         self.iMax = min(15, self.iTotal - self.iOpeningDraw)
-        self.iDrawStep = 1 # Increments to use in table
+        self.iDrawStep = 1  # Increments to use in table
         self.aAllChoices = _gen_choice_list(self.dSelectedCounts)
         self.iCardsToDraw = min(3, self.iSelectedCount)
 
         if self.iTotal <= self.iOpeningDraw:
             if bLibrary:
                 do_complaint_error("Library must be larger than the opening"
-                        "hand")
+                        " hand")
             else:
                 do_complaint_error("Crypt must be larger than"
                         " the opening draw")
@@ -185,11 +199,9 @@ class CardDrawSimPlugin(CardListPlugin):
 
         oDialog.run()
 
-    # pylint: enable-msg=W0613, W0201
-
     def _make_dialog(self, oMainTitle, bCrypt):
         """Create the dialog box."""
-        oDialog = SutekhDialog( "Card Draw probablities", self.parent,
+        oDialog = SutekhDialog("Card Draw probablities", self.parent,
             gtk.DIALOG_DESTROY_WITH_PARENT,
             (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
         oDialog.connect("response", lambda oDialog, resp: oDialog.destroy())
@@ -259,7 +271,6 @@ class CardDrawSimPlugin(CardListPlugin):
         oDialog.vbox.pack_start(oResultsBox)
         return oDialog
 
-
     def _setup_cardlists(self, aSelectedCards, bCrypt):
         """Extract the needed card info from the model"""
         aAllAbsCards = [IAbstractCard(oCard) for oCard in
@@ -268,14 +279,16 @@ class CardDrawSimPlugin(CardListPlugin):
         iLibrarySize = 0
         self.dSelectedCounts = {}
         self.iSelectedCount = 0
+        # Initialise dict 1st, as cards with a count of 0 in the selection
+        # are possible.
+        for oCard in aSelectedCards:
+            self.dSelectedCounts.setdefault(oCard, 0)
         for oCard in aAllAbsCards:
-            aTypes = [oType.name for oType in oCard.cardtype]
-            if aTypes[0] == 'Vampire' or aTypes[0] == 'Imbued':
+            if is_crypt_card(oCard):
                 iCryptSize += 1
             else:
                 iLibrarySize += 1
-            if oCard in aSelectedCards:
-                self.dSelectedCounts.setdefault(oCard, 0)
+            if oCard in self.dSelectedCounts:
                 self.dSelectedCounts[oCard] += 1
                 self.iSelectedCount += 1
             # The assumption is that the user is interested in all copies of
@@ -292,15 +305,12 @@ class CardDrawSimPlugin(CardListPlugin):
         aSelectedCards = []
         bCrypt = False
         bLibrary = False
-        # pylint: disable-msg=W0612
-        # oModel will be unused
-        oModel, aSelection = self.view.get_selection().get_selected_rows()
+        _oModel, aSelection = self.view.get_selection().get_selected_rows()
         for oPath in aSelection:
             # pylint: disable-msg=E1101
             # pylint doesn't pick up adaptor's methods correctly
             oCard = IAbstractCard(self.model.get_card_name_from_path(oPath))
-            aTypes = [oType.name for oType in oCard.cardtype]
-            if aTypes[0] == 'Vampire' or aTypes[0] == 'Imbued':
+            if is_crypt_card(oCard):
                 bCrypt = True
             else:
                 bLibrary = True
@@ -320,9 +330,7 @@ class CardDrawSimPlugin(CardListPlugin):
         """Handle changes to the oNumDraws combo box."""
         self.iNumSteps = int(oComboBox.get_active_text())
 
-    # pylint: disable-msg=W0613
-    # oWidget is intentionally unused
-    def _fill_table(self, oWidget, bCrypt):
+    def _fill_table(self, _oWidget, bCrypt):
         """Fill Results Box with the draw results.
 
            oWidget is a dummy placeholder so this method can be called by the
@@ -386,7 +394,6 @@ class CardDrawSimPlugin(CardListPlugin):
         for iRow in range(1, iNumCardRows):
             self._fill_row(iRow, iOffset, False, aCardCounts)
         self.oResultsTable.show_all()
-    # pylint: enable-msg=W0613
 
     def _setup_table(self, iNumRows, iNumCols):
         """Clear the table, and setup constant entries."""
@@ -402,7 +409,7 @@ class CardDrawSimPlugin(CardListPlugin):
         self.oResultsTable.attach(oLeftLabel, 0, 1, 1, iNumRows)
 
     def _setup_count_rows(self):
-        """Fill in the culmative count rows."""
+        """Fill in the cumulative count rows."""
         iBottomRow = 3
         for iCardRow in range(self.iCardsToDraw + 1):
             self.oResultsTable.attach(gtk.HSeparator(), 1, 3, iBottomRow,
@@ -412,7 +419,7 @@ class CardDrawSimPlugin(CardListPlugin):
             iNumRows = len([x for x in self.aAllChoices if
                 sum(x) == iCardRow])
             iTopRow = iBottomRow + 2 * iNumRows
-            self.oResultsTable.attach(oLabel, 2 , 3, iBottomRow + 1, iTopRow)
+            self.oResultsTable.attach(oLabel, 2, 3, iBottomRow + 1, iTopRow)
             iBottomRow = iTopRow
 
     def _fill_row(self, iRow, iOffset, bZero, aCardCounts):
@@ -423,10 +430,10 @@ class CardDrawSimPlugin(CardListPlugin):
             iNumDraws = iCol * self.iDrawStep + self.iOpeningDraw
             if iNumDraws < self.iTotal:
                 fProbExact = _multi_hyper_prob(aThisDraw, iNumDraws,
-                        aCardCounts, self.iTotal)*100
+                        aCardCounts, self.iTotal) * 100
                 if not bZero:
                     fProbAccum = _hyper_prob_at_least(aThisDraw, iNumDraws,
-                            aCardCounts, self.iTotal)*100
+                            aCardCounts, self.iTotal) * 100
                     oResLabel = gtk.Label('%3.2f (%3.2f)' % (fProbAccum,
                         fProbExact))
                 else:
@@ -454,6 +461,4 @@ class CardDrawSimPlugin(CardListPlugin):
             sLabel += u'%d \u00D7 %s\n' % (iCount, oItem[0].name)
         return sLabel
 
-# pylint: disable-msg=C0103
-# plugin name doesn't match, but ignore that
 plugin = CardDrawSimPlugin
