@@ -53,7 +53,7 @@ class StarterConfigDialog(SutekhDialog):
                     ' plugin</b>\nChoose cancel to skip configuring the '
                     ' plugin\nYou will not be prompted again')
         self.oFileWidget = FileOrUrlWidget(oParent, "Choose localtion for "
-                "Starter decks", {'Sutekh Wiki': self.sDocUrl})
+                "Starter decks", {'Sutekh Datapack': self.sDocUrl})
         add_filter(self.oFileWidget, 'Zip Files', ['*.zip', '*.ZIP'])
         # pylint: disable-msg=E1101
         # pylint doesn't pick up vbox methods correctly
@@ -95,8 +95,8 @@ class StarterConfigDialog(SutekhDialog):
         sFile, _bUrl = self.oFileWidget.get_file_or_url()
         sData = None
         if sFile == self.sDocUrl:
-            # Downloading from sutekh wiki, so need magic to get right file
-            sZipUrl = find_data_pack('starters')
+            # Downloading from sutekh datapack, so need magic to get right file
+            sZipUrl, _ = find_data_pack('starters')
             oFile = urllib2.urlopen(sZipUrl)
             sData = fetch_data(oFile)
         elif sFile:
@@ -247,6 +247,7 @@ class StarterInfoPlugin(SutekhPlugin, CardTextViewListener):
         if not bOK:
             oProgressDialog.destroy()
             return False  # No starters in zip file
+        sRemovedInfo = oFile.get_info_file('removed_decks.txt')
         oLogger.addHandler(oLogHandler)
         oLogHandler.set_dialog(oProgressDialog)
         oLogHandler.set_total(len(dList))
@@ -264,6 +265,7 @@ class StarterInfoPlugin(SutekhPlugin, CardTextViewListener):
                 return False  # Error
         # Cleanup
         self._clean_empty(oFile.get_all_entries(), aExistingList)
+        self._clean_removed(sRemovedInfo)
         self.reload_pcs_list()
         oProgressDialog.destroy()
         return True
@@ -327,6 +329,27 @@ class StarterInfoPlugin(SutekhPlugin, CardTextViewListener):
                 return False
         return True
 
+    def _clean_removed(self, sRemovedInfo):
+        """Handle card sets that are marked as removed in the starter
+           info file."""
+        if not sRemovedInfo:
+            # No removed info, so skip
+            return
+        for sLine in sRemovedInfo.splitlines():
+            if sLine.startswith('#'):
+                # Ignore comment
+                continue
+            sName = sLine.strip()
+            try:
+                # Removed deck still present in the database, so delete it
+                oCS = IPhysicalCardSet(sName)
+                if has_children(oCS):
+                    # FIXME: Prompt in this case
+                    continue
+                delete_physical_card_set(sName)
+            except SQLObjectNotFound:
+                pass
+
     # pylint: disable-msg=R0201
     # Method for consistency with _unzip methods
     def _clean_empty(self, aMyList, aExistingList):
