@@ -9,10 +9,11 @@
 from sutekh.tests.TestCore import SutekhTest
 from sutekh.core.SutekhObjects import AbstractCard, IAbstractCard, \
         IPhysicalCard, IClan, IDisciplinePair, ICardType, ISect, ITitle, \
-        ICreed, IVirtue, IExpansion, IRarity, IRarityPair
+        ICreed, IVirtue, IExpansion, IRarity, IRarityPair, IArtist, IKeyword
 from sqlobject import SQLObjectNotFound
 import unittest
 
+
 class WhiteWolfParserTests(SutekhTest):
     """Test class for the white wolf card reader.
 
@@ -20,20 +21,77 @@ class WhiteWolfParserTests(SutekhTest):
        are correct.
        """
     aExpectedCards = [
-        u".44 Magnum", u"AK-47", u"Aabbt Kindred",
-        u"Aaron Bathurst", u"Aaron Duggan, Cameron's Toady",
-        u"Aaron's Feeding Razor", u"Abandoning the Flesh",
-        u"Abbot", u"Abd al-Rashid", u"Abdelsobek",
-        u"Abebe", u"Abjure", u"Ablative Skin",
-        u"Abombwe", u"Aeron", u"Aire of Elation", u"Akram",
-        u"Alan Sovereign", u"Alan Sovereign (Advanced)", u"Alexandra",
-        u"Alfred Benezri", u"Ambrogino Giovanni", u'Amisa',
-        u"Anastasz di Zagreb", u"Angelica, The Canonicus",
-        u'Anna "Dictatrix11" Suljic', u"Anson", u"Bronwen", u"Cedric",
-        u"Cesewayo", u'Earl "Shaka74" Deams', u"Gracis Nostinus",
-        u'Inez "Nurse216" Villagrande', u"Kabede Maru",
-        u"Kemintiri (Advanced)", u"L\xe1z\xe1r Dobrescu",
-        u"Predator's Communion", u"Sha-Ennu", u"The Path of Blood",
+        u".44 Magnum",
+        u"AK-47",
+        u"Aabbt Kindred",
+        u"Aaron Bathurst",
+        u"Aaron Duggan, Cameron's Toady",
+        u"Aaron's Feeding Razor",
+        u"Abandoning the Flesh",
+        u"Abbot",
+        u"Abd al-Rashid",
+        u"Abdelsobek",
+        u"Abebe",
+        u"Abjure",
+        u"Ablative Skin",
+        u"Abombwe",
+        u"Aeron",
+        u"Agent of Power",
+        u"Aire of Elation",
+        u"Akram",
+        u"Alan Sovereign",
+        u"Alan Sovereign (Advanced)",
+        u"Alexandra",
+        u"Alfred Benezri",
+        u"Ambrogino Giovanni",
+        u'Amisa',
+        u'Anarch Railroad',
+        u'Anarch Revolt',
+        u"Anastasz di Zagreb",
+        u"Angelica, The Canonicus",
+        u'Anna "Dictatrix11" Suljic',
+        u"Anson",
+        u"Bravo",
+        u"Bronwen",
+        u"Cedric",
+        u"Cesewayo",
+        u'Dramatic Upheaval',
+        u'Earl "Shaka74" Deams',
+        u"Enkidu, The Noah",
+        u"Fidus, The Shrunken Beast",
+        u"Ghoul Retainer",
+        u"Gracis Nostinus",
+        u"Gypsies",
+        u'Hide the Heart',
+        u"High Top",
+        u'Inez "Nurse216" Villagrande',
+        u"Kabede Maru",
+        u"Kemintiri (Advanced)",
+        u"Living Manse",
+        u"L\xe1z\xe1r Dobrescu",
+        u'Motivated by Gehenna',
+        u"Necromancy",
+        u"Ossian",
+        u"Pariah",
+        u"Paris Opera House",
+        u"Park Hunting Ground",
+        u"Pier 13, Port of Baltimore",
+        u"Political Hunting Ground",
+        u"Predator's Communion",
+        u"Protracted Investment",
+        u"Raven Spy",
+        u"Rebekka, Chantry Elder of Munich",
+        u"Rock Cat",
+        u'Scapelli, The Family "Mechanic"',
+        u"Sha-Ennu",
+        u"Shade",
+        u"Smite",
+        u"The Ankara Citadel, Turkey",
+        u"The Path of Blood",
+        u"The Siamese",
+        u"The Slaughterhouse",
+        u"Two Wrongs",
+        u"Vox Domini",
         u"Yvette, The Hopeless",
         ]
 
@@ -74,6 +132,8 @@ class WhiteWolfParserTests(SutekhTest):
 
         self.assertTrue(IRarityPair(('VTES', 'Common')) in o44.rarity)
 
+        self.failUnless(IKeyword('gun') in o44.keywords)
+        self.failUnless(IKeyword('weapon') in o44.keywords)
 
         # Find some discipline pairs
         oFortInf = IDisciplinePair((u"Fortitude", u"inferior"))
@@ -86,6 +146,7 @@ class WhiteWolfParserTests(SutekhTest):
         oObfSup = IDisciplinePair((u"Obfuscate", u"superior"))
         oAboInf = IDisciplinePair((u"Abombwe", u"inferior"))
         oAboSup = IDisciplinePair((u"Abombwe", u"superior"))
+        oValSup = IDisciplinePair((u'Valeren', u"superior"))
 
         # Check Dobrescu
         oDob = IAbstractCard(u"L\xe1z\xe1r Dobrescu")
@@ -107,12 +168,14 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertEqual(len(oDob.cardtype), 1)
         self.failUnless(ICardType('Vampire') in oDob.cardtype)
         self.failUnless(ISect('Independent') in oDob.sect)
+        self.assertEqual(len(oDob.artists), 1)
+        self.failUnless(IArtist('Rebecca Guay') in oDob.artists)
 
         self.assertEqual(len(oDob.rulings), 1)
         oRuling = oDob.rulings[0]
         self.assertTrue(oRuling.text.startswith("Cannot use his special"))
         self.assertTrue(oRuling.text.endswith("uncontrolled region."))
-        self.assertEqual(oRuling.code, "[LSJ19990215]")
+        self.assertEqual(oRuling.code, "[LSJ 19990215]")
 
         # Check Abstract and Physical expansions match
         for oAbs in AbstractCard.select():
@@ -121,10 +184,8 @@ class WhiteWolfParserTests(SutekhTest):
                 try:
                     oPair = IPhysicalCard((oAbs, oExp))
                 except SQLObjectNotFound:
-                    self.fail(
-                        "Missing physical card %s from expansion %s"
-                        % (oAbs.name, oExp.name)
-                    )
+                    self.fail("Missing physical card %s from expansion %s"
+                        % (oAbs.name, oExp.name))
 
         # Check Yvette
         oYvette = IAbstractCard(u"Yvette, The Hopeless")
@@ -147,6 +208,8 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertEqual(len(oYvette.cardtype), 1)
         self.failUnless(ICardType('Vampire') in oYvette.cardtype)
         self.failUnless(ISect('Camarilla') in oYvette.sect)
+        self.assertEqual(len(oYvette.artists), 1)
+        self.failUnless(IArtist('Leif Jones') in oYvette.artists)
 
         # Check Sha-Ennu
 
@@ -174,6 +237,13 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertTrue(IRarityPair(('Third', 'Vampire')) in oShaEnnu.rarity)
         self.assertFalse(IRarityPair(('VTES', 'Common')) in oShaEnnu.rarity)
         self.assertFalse(IRarityPair(('VTES', 'Vampire')) in oShaEnnu.rarity)
+        self.assertEqual(len(oShaEnnu.artists), 1)
+        self.failUnless(IArtist('Richard Thomas') in oShaEnnu.artists)
+
+        self.failUnless(IKeyword('0 stealth') in oShaEnnu.keywords)
+        self.failUnless(IKeyword('0 intercept') in oShaEnnu.keywords)
+        self.failUnless(IKeyword('1 strength') in oShaEnnu.keywords)
+        self.failUnless(IKeyword('3 bleed') in oShaEnnu.keywords)
 
         # Check Kabede
 
@@ -201,6 +271,13 @@ class WhiteWolfParserTests(SutekhTest):
         self.failUnless(ISect('Laibon') in oKabede.sect)
         self.failUnless(ITitle('Magaji') in oKabede.title)
         self.assertTrue(IRarityPair(('LotN', 'Uncommon')) in oKabede.rarity)
+        self.assertEqual(len(oKabede.artists), 1)
+        self.failUnless(IArtist('Ken Meyer, Jr.') in oKabede.artists)
+
+        self.failUnless(IKeyword('0 stealth') in oKabede.keywords)
+        self.failUnless(IKeyword('0 intercept') in oKabede.keywords)
+        self.failUnless(IKeyword('1 strength') in oKabede.keywords)
+        self.failUnless(IKeyword('1 bleed') in oKabede.keywords)
 
         # Check Predator's Communion
         oPredComm = IAbstractCard(u"Predator's Communion")
@@ -221,6 +298,8 @@ class WhiteWolfParserTests(SutekhTest):
         self.failUnless(ICardType('Reaction') in oPredComm.cardtype)
         self.failUnless(ICardType('Reflex') in oPredComm.cardtype)
         self.assertTrue(IRarityPair(('LoB', 'Common')) in oPredComm.rarity)
+        self.assertEqual(len(oPredComm.artists), 1)
+        self.failUnless(IArtist('David Day') in oPredComm.artists)
 
         # Check Earl
         oEarl = IAbstractCard(u'Earl "Shaka74" Deams')
@@ -244,6 +323,8 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertEqual(len(oEarl.cardtype), 1)
         self.failUnless(ICardType('Imbued') in oEarl.cardtype)
         self.assertTrue(IRarityPair(('NoR', 'Uncommon')) in oEarl.rarity)
+        self.assertEqual(len(oEarl.artists), 1)
+        self.failUnless(IArtist('David Day') in oEarl.artists)
 
         # Check Aire
         oAire = IAbstractCard("Aire of Elation")
@@ -270,6 +351,9 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertTrue(IRarityPair(('Anarchs', 'Precon')) in oAire.rarity)
         self.assertTrue(IRarityPair(('KMW', 'Precon')) in oAire.rarity)
 
+        self.assertEqual(len(oAire.artists), 1)
+        self.failUnless(IArtist('Greg Simanson') in oAire.artists)
+
         # Abjure
         oAbjure = IAbstractCard("Abjure")
         self.assertEqual(oAbjure.canonicalName, u"abjure")
@@ -287,6 +371,8 @@ class WhiteWolfParserTests(SutekhTest):
         self.failUnless(IVirtue('Redemption') in oAbjure.virtue)
         self.assertEqual(len(oAbjure.cardtype), 1)
         self.failUnless(ICardType('Power') in oAbjure.cardtype)
+        self.assertEqual(len(oAbjure.artists), 1)
+        self.failUnless(IArtist('Brian LeBlanc') in oAbjure.artists)
 
         # Abombwe Master Card
         oAbo = IAbstractCard('Abombwe')
@@ -306,6 +392,11 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertEqual(len(oAbo.cardtype), 1)
         self.failUnless(ICardType('Master') in oAbo.cardtype)
         self.assertEqual(len(oAbo.rulings), 0)
+        self.assertEqual(len(oAbo.artists), 1)
+        self.failUnless(IArtist('Ken Meyer, Jr.') in oAbo.artists)
+
+        self.failUnless(IKeyword('trifle') in oAbo.keywords)
+        self.failUnless(IKeyword('discipline') in oAbo.keywords)
 
         # Ablative Skin Card
         oAblat = IAbstractCard('Ablative Skin')
@@ -328,7 +419,9 @@ class WhiteWolfParserTests(SutekhTest):
         oRuling = oAblat.rulings[0]
         self.assertTrue(oRuling.text.startswith("Cannot be used to prevent"))
         self.assertTrue(oRuling.text.endswith("Blood Fury)."))
-        self.assertEqual(oRuling.code, "[LSJ19990216]")
+        self.assertEqual(oRuling.code, "[LSJ 19990216]")
+        self.assertEqual(len(oAblat.artists), 1)
+        self.failUnless(IArtist('Richard Thomas') in oAblat.artists)
 
         # Check Independent titles
         oAmbrig = IAbstractCard(u"Ambrogino Giovanni")
@@ -347,6 +440,7 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertEqual(oKemintiri.life, None)
         self.assertEqual(oKemintiri.costtype, None)
         self.assertEqual(oKemintiri.level, 'advanced')
+        self.failUnless(IKeyword('advanced') in oKemintiri.keywords)
 
         self.failUnless(IClan('Follower of Set') in oKemintiri.clan)
         self.assertEqual(len(oKemintiri.clan), 1)
@@ -357,10 +451,119 @@ class WhiteWolfParserTests(SutekhTest):
         self.assertEqual(len(oKemintiri.cardtype), 1)
         self.failUnless(ICardType('Vampire') in oKemintiri.cardtype)
         self.failUnless(ISect('Independent') in oKemintiri.sect)
+        self.assertEqual(len(oKemintiri.artists), 1)
+        self.failUnless(IArtist('Lawrence Snelly') in oKemintiri.artists)
+
         # Make sure we're not picking up the Merged title
         self.assertEqual(len(oKemintiri.title), 0)
         self.assertTrue(IRarityPair(('KMW', 'Uncommon')) in oKemintiri.rarity)
 
+        # Check The Path
+        oPath1 = IAbstractCard('The Path of Blood')
+
+        self.failUnless(IClan('Assamite') in oPath1.clan)
+        self.assertEqual(oPath1.cost, 1)
+        self.assertEqual(oPath1.costtype, 'pool')
+        self.failUnless(ICardType('Master') in oPath1.cardtype)
+
+        self.failUnless(IKeyword('unique') in oPath1.keywords)
+
+        # Check alternative lookup
+        oPath2 = IAbstractCard('Path of Blood, The')
+        self.assertEqual(oPath1.id, oPath2.id)
+
+        # Check The Slaughterhouse
+        oSlaughter = IAbstractCard('The Slaughterhouse')
+        self.assertEqual(oSlaughter.canonicalName, u"the slaughterhouse")
+        self.assertEqual(len(oSlaughter.keywords), 2)
+        self.failUnless(IKeyword('burn option') in oSlaughter.keywords)
+        self.failUnless(IKeyword('location') in oSlaughter.keywords)
+
+        # Check keywords for The Ankara Citadel, Turkey
+        oCitadel = IAbstractCard('The Ankara Citadel, Turkey')
+        self.assertEqual(oCitadel.cost, 2)
+        self.failUnless(IClan('Tremere') in oCitadel.clan)
+        self.assertEqual(oCitadel.costtype, 'blood')
+        self.failUnless(IKeyword('location') in oCitadel.keywords)
+        self.failUnless(IKeyword('unique') in oCitadel.keywords)
+
+        # Check life & keywords for Ossian
+        oOssian = IAbstractCard('Ossian')
+        self.assertEqual(oOssian.life, 4)
+        self.assertEqual(oOssian.cost, 3)
+        self.assertEqual(oOssian.costtype, 'pool')
+        self.failUnless(IKeyword('werewolf') in oOssian.keywords)
+        self.failUnless(IKeyword('red list') in oOssian.keywords)
+        self.failUnless(IKeyword('unique') in oOssian.keywords)
+        self.failUnless(IKeyword('0 bleed') in oOssian.keywords)
+        self.failUnless(IKeyword('2 strength') in oOssian.keywords)
+
+        # Check life & keywords for Raven Spy
+        oRaven = IAbstractCard('Raven Spy')
+        self.failUnless(IKeyword('animal') in oRaven.keywords)
+        self.failUnless(IKeyword('1 strength') not in oRaven.keywords)
+        self.assertEqual(oRaven.life, 1)
+        self.assertEqual(oRaven.cost, 1)
+        self.assertEqual(oRaven.costtype, 'blood')
+
+        # Check special cases
+        oRetainer = IAbstractCard('Ghoul Retainer')
+        self.failUnless(IKeyword('1 strength') in oRetainer.keywords)
+        oHighTop = IAbstractCard('High Top')
+        self.failUnless(IKeyword('1 intercept') in oHighTop.keywords)
+        oGypsies = IAbstractCard('Gypsies')
+        self.failUnless(IKeyword('1 stealth') in oGypsies.keywords)
+        oRebekka = IAbstractCard('Rebekka, Chantry Elder of Munich')
+        self.failUnless(IKeyword('1 stealth') in oRebekka.keywords)
+        oProtracted = IAbstractCard('Protracted Investment')
+        self.failUnless(IKeyword('investment') in oProtracted.keywords)
+        oBravo = IAbstractCard('Bravo')
+        self.failUnless(IKeyword('archetype') in oBravo.keywords)
+        oDramatic = IAbstractCard('Dramatic Upheaval')
+        self.failUnless(IKeyword('not for legal play') in oDramatic.keywords)
+        oMotivated = IAbstractCard('Motivated by Gehenna')
+        self.failUnless(IKeyword('not for legal play') in oMotivated.keywords)
+
+        # Test adding extra expansions works
+
+        oAnarchRailroad = IAbstractCard("Anarch Railroad")
+        self.assertEqual(oAnarchRailroad.canonicalName, u"anarch railroad")
+        self.assertEqual(oAnarchRailroad.cost, 2)
+        self.assertEqual(oAnarchRailroad.costtype, 'pool')
+        self.failUnless(oAnarchRailroad.text.startswith(u"Master: unique"))
+        oAnarchs = IExpansion('Anarchs')
+        oAA = IExpansion('Anarchs and Alastors Storyline')
+
+        self.assertTrue(oAnarchs in [oP.expansion for oP in
+            oAnarchRailroad.rarity])
+        self.assertTrue(oAA in [oP.expansion for oP in oAnarchRailroad.rarity])
+
+        oAnarchRevolt = IAbstractCard("Anarch Revolt")
+        self.assertEqual(oAnarchRevolt.canonicalName, u"anarch revolt")
+        self.assertEqual(oAnarchRevolt.cost, None)
+        self.failUnless(oAnarchRevolt.text.startswith(u"Master."))
+
+        self.assertTrue(oAnarchs in [oP.expansion for oP in
+            oAnarchRevolt.rarity])
+        self.assertTrue(oAA in [oP.expansion for oP in oAnarchRevolt.rarity])
+        self.assertTrue(oJyhad in [oP.expansion for oP in
+            oAnarchRevolt.rarity])
+
+        oHtH = IAbstractCard("Hide the Heart")
+        self.assertEqual(oHtH.canonicalName, u"hide the heart")
+        self.assertEqual(len(oHtH.discipline), 2)
+        self.failUnless(oAusSup in oHtH.discipline)
+        self.failUnless(oValSup in oHtH.discipline)
+        self.assertEqual(len(oHtH.cardtype), 1)
+        self.failUnless(ICardType("Reaction") in oHtH.cardtype)
+        self.failUnless(oHtH.text.startswith('[aus] Reduce'))
+
+        oSmite = IAbstractCard("Smite")
+        self.assertEqual(oSmite.canonicalName, u"smite")
+        self.assertEqual(oSmite.cost, 3)
+        self.assertEqual(oSmite.costtype, "conviction")
+        self.failUnless(oSmite.text.startswith('{Strike:}'))
+
 
 if __name__ == "__main__":
     unittest.main()
